@@ -1,14 +1,15 @@
 """Discovers Chromecasts on the network using mDNS/zeroconf."""
+import socket
 from uuid import UUID
 
-import six
-from zeroconf import ServiceBrowser, Zeroconf
+import zeroconf
 
 DISCOVER_TIMEOUT = 5
 
 
 class CastListener(object):
     """Zeroconf Cast Services collection."""
+
     def __init__(self, callback=None):
         self.services = {}
         self.callback = callback
@@ -45,10 +46,10 @@ class CastListener(object):
             return
 
         def get_value(key):
-            """Retrieve value and decode for Python 2/3."""
+            """Retrieve value and decode to UTF-8."""
             value = service.properties.get(key.encode('utf-8'))
 
-            if value is None or isinstance(value, six.text_type):
+            if value is None or isinstance(value, str):
                 return value
             return value.decode('utf-8')
 
@@ -84,8 +85,19 @@ def start_discovery(callback=None):
     ServiceBrowser object.
     """
     listener = CastListener(callback)
-    return listener, \
-        ServiceBrowser(Zeroconf(), "_googlecast._tcp.local.", listener)
+    service_browser = False
+    try:
+        service_browser = zeroconf.ServiceBrowser(zeroconf.Zeroconf(),
+                                                  "_googlecast._tcp.local.",
+                                                  listener)
+    except (zeroconf.BadTypeInNameException,
+            NotImplementedError,
+            OSError,
+            socket.error,
+            zeroconf.NonUniqueNameException):
+        pass
+
+    return listener, service_browser
 
 
 def stop_discovery(browser):
@@ -96,6 +108,7 @@ def stop_discovery(browser):
 def discover_chromecasts(max_devices=None, timeout=DISCOVER_TIMEOUT):
     """ Discover chromecasts on the network. """
     from threading import Event
+    browser = False
     try:
         # pylint: disable=unused-argument
         def callback(name):
@@ -110,5 +123,8 @@ def discover_chromecasts(max_devices=None, timeout=DISCOVER_TIMEOUT):
         discover_complete.wait(timeout)
 
         return listener.devices
+    except Exception:  # pylint: disable=broad-except
+        raise
     finally:
-        stop_discovery(browser)
+        if browser is not False:
+            stop_discovery(browser)
