@@ -8,21 +8,22 @@ Without him this would not have been possible.
 # Pylint does not understand the protobuf objects correctly
 # pylint: disable=no-member
 
+import errno
+import json
 import logging
 import select
 import socket
 import ssl
-import json
+import sys
 import threading
 import time
 from collections import namedtuple
 from struct import pack, unpack
-import sys
 
 from . import cast_channel_pb2
-from .dial import CAST_TYPE_CHROMECAST, CAST_TYPE_AUDIO, CAST_TYPE_GROUP
 from .controllers import BaseController
 from .controllers.media import MediaController
+from .dial import CAST_TYPE_CHROMECAST, CAST_TYPE_AUDIO, CAST_TYPE_GROUP
 from .error import (
     ChromecastConnectionError,
     UnsupportedNamespace,
@@ -117,18 +118,15 @@ def _is_ssl_timeout(exc):
 NetworkAddress = namedtuple('NetworkAddress',
                             ['address', 'port'])
 
-
 ConnectionStatus = namedtuple('ConnectionStatus',
                               ['status', 'address'])
 
-
 CastStatus = namedtuple('CastStatus',
                         ['is_active_input', 'is_stand_by', 'volume_level',
                          'volume_muted', 'app_id', 'display_name',
                          'namespaces', 'session_id', 'transport_id',
                          'status_text'])
 
-
 LaunchFailure = namedtuple('LaunchStatus',
                            ['reason', 'app_id', 'request_id'])
 
@@ -238,12 +236,13 @@ class SocketClient(threading.Thread):
 
         while not self.stop.is_set() and (tries is None or tries > 0):
             try:
-                self.socket = ssl.wrap_socket(socket.socket())
+                self.socket = new_socket()
                 self.socket.settimeout(self.timeout)
                 self._report_connection_status(
                     ConnectionStatus(CONNECTION_STATUS_CONNECTING,
                                      NetworkAddress(self.host, self.port)))
                 self.socket.connect((self.host, self.port))
+                self.socket = ssl.wrap_socket(self.socket)
                 self.connecting = False
                 self._force_recon = False
                 self._report_connection_status(
@@ -255,7 +254,7 @@ class SocketClient(threading.Thread):
 
                 self.logger.debug("Connected!")
                 break
-            except socket.error as err:
+            except OSError as err:
                 self.connecting = True
                 if self.stop.is_set():
                     self.logger.error(
@@ -266,11 +265,14 @@ class SocketClient(threading.Thread):
                 self._report_connection_status(
                     ConnectionStatus(CONNECTION_STATUS_FAILED,
                                      NetworkAddress(self.host, self.port)))
-                retry_log_fun("Failed to connect, retrying in %.1fs",
-                              self.retry_wait)
-                retry_log_fun = self.logger.debug
 
-                time.sleep(self.retry_wait)
+                # Only sleep if we have another retry remaining
+                if tries is None or tries > 1:
+                    retry_log_fun("Failed to connect, retrying in %.1fs",
+                                  self.retry_wait)
+                    retry_log_fun = self.logger.debug
+                    time.sleep(self.retry_wait)
+
                 if tries:
                     tries -= 1
         else:
@@ -293,7 +295,7 @@ class SocketClient(threading.Thread):
         new_channel = self.destination_id != cast_status.transport_id
 
         if new_channel:
-            self._disconnect_channel(self.destination_id)
+            self.disconnect_channel(self.destination_id)
 
         self.app_namespaces = cast_status.namespaces
         self.destination_id = cast_status.transport_id
@@ -347,7 +349,7 @@ class SocketClient(threading.Thread):
         Use run_once() in your own main loop after you
         receive something on the socket (get_socket()).
         """
-        # pylint: disable=too-many-branches
+        # pylint: disable=too-many-branches, too-many-return-statements
 
         try:
             if not self._check_connection():
@@ -403,6 +405,8 @@ class SocketClient(threading.Thread):
                 if function:
                     function(data)
 
+        return 0
+
     def get_socket(self):
         """
         Returns the socket of the connection to use it in you own
@@ -462,8 +466,8 @@ class SocketClient(threading.Thread):
                             _message_to_string(message, data))
             except Exception:  # pylint: disable=broad-except
                 self.logger.exception(
-                    (u"Exception caught while sending message to "
-                     u"controller %s: %s"),
+                    ("Exception caught while sending message to "
+                     "controller %s: %s"),
                     type(self._handlers[message.namespace]).__name__,
                     _message_to_string(message, data))
 
@@ -476,7 +480,7 @@ class SocketClient(threading.Thread):
         """ Cleanup open channels and handlers """
         for channel in self._open_channels:
             try:
-                self._disconnect_channel(channel)
+                self.disconnect_channel(channel)
             except Exception:  # pylint: disable=broad-except
                 pass
 
@@ -582,20 +586,20 @@ class SocketClient(threading.Thread):
             raise PyChromecastStopped("Socket client's thread is stopped.")
         if not self.connecting and not self._force_recon:
             try:
+                if not no_add_request_id and callback_function:
+                    self._request_callbacks[request_id] = {
+                        'event': threading.Event(),
+                        'response': None,
+                        'function': callback_function,
+                    }
                 self.socket.sendall(be_size + msg.SerializeToString())
             except socket.error:
+                self._request_callbacks.pop(request_id, None)
                 self._force_recon = True
                 self.logger.info('Error writing to socket.')
         else:
             raise NotConnected("Chromecast is connecting...")
 
-        if not no_add_request_id and callback_function:
-            self._request_callbacks[request_id] = {
-                'event': threading.Event(),
-                'response': None,
-                'function': callback_function,
-            }
-
     def send_platform_message(self, namespace, message, inc_session_id=False,
                               callback_function_param=False):
         """ Helper method to send a message to the platform. """
@@ -639,7 +643,7 @@ class SocketClient(threading.Thread):
                      'connectionType': 1}},
                 no_add_request_id=True)
 
-    def _disconnect_channel(self, destination_id):
+    def disconnect_channel(self, destination_id):
         """ Disconnect a channel with destination_id. """
         if destination_id in self._open_channels:
             self.send_message(
@@ -674,12 +678,15 @@ class ConnectionController(BaseController):
             return True
 
         if data[MESSAGE_TYPE] == TYPE_CLOSE:
-            self._socket_client.handle_channel_disconnected()
+            # The cast device is asking us to acknowledge closing this channel.
+            self._socket_client.disconnect_channel(message.source_id)
+
+            # Schedule a status update so that a channel is created.
+            self._socket_client.receiver_controller.update_status()
 
             return True
 
-        else:
-            return False
+        return False
 
 
 class HeartbeatController(BaseController):
@@ -712,8 +719,7 @@ class HeartbeatController(BaseController):
             self.reset()
             return True
 
-        else:
-            return False
+        return False
 
     def ping(self):
         """ Send a ping message. """
@@ -775,8 +781,7 @@ class ReceiverController(BaseController):
 
             return True
 
-        else:
-            return False
+        return False
 
     def register_status_listener(self, listener):
         """ Register a status listener for when a new Chromecast status
@@ -892,7 +897,7 @@ class ReceiverController(BaseController):
             app_data.get(SESSION_ID),
             app_data.get('transportId'),
             app_data.get('statusText', '')
-            )
+        )
         return status
 
     def _process_get_status(self, data):
@@ -964,3 +969,29 @@ class ReceiverController(BaseController):
         self._report_status()
 
         self._status_listeners[:] = []
+
+
+def new_socket():
+    """
+    Create a new socket with OS-specific parameters
+
+    Try to set SO_REUSEPORT for BSD-flavored systems if it's an option.
+    Catches errors if not.
+    """
+    new_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    new_sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+
+    try:
+        # noinspection PyUnresolvedReferences
+        reuseport = socket.SO_REUSEPORT
+    except AttributeError:
+        pass
+    else:
+        try:
+            new_sock.setsockopt(socket.SOL_SOCKET, reuseport, 1)
+        except (OSError, socket.error) as err:
+            # OSError on python 3, socket.error on python 2
+            if err.errno != errno.ENOPROTOOPT:
+                raise
+
+    return new_sock
